@@ -1,5 +1,6 @@
 use crate::{
-    ByteSource, ByteUnsealable, CryptoError, Data, DataBuilder, HasIndex, Key, KeyBuilder,
+    ByteSource, ByteUnsealable, CryptoError, Data, DataBuilder, HasByteSource, HasIndex, Key,
+    KeyBuilder,
 };
 use mongodb::bson::Document;
 use serde::{Deserialize, Serialize};
@@ -29,6 +30,15 @@ impl Entry {
     }
 }
 
+impl<T: HasBuilder + HasByteSource> From<T> for States {
+    fn from(value: T) -> Self {
+        States::Unsealed {
+            builder: value.builder().into(),
+            bytes: value.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum States {
@@ -52,10 +62,10 @@ pub trait HasBuilder {
     fn builder(&self) -> Self::Builder;
 }
 
-pub trait Builder: TryFrom<TypeBuilderContainer, Error = CryptoError> {
+pub trait Builder: TryFrom<TypeBuilderContainer, Error = CryptoError> + Into<TypeBuilder> {
     type Output;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError>;
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError>;
 }
 
 /// Need this to provide a level an indirection for TryFrom
@@ -106,7 +116,7 @@ impl TryFrom<TypeBuilderContainer> for TypeBuilder {
 impl Builder for TypeBuilder {
     type Output = Type;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::Key(k) => Ok(Type::Key(k.build(bytes)?)),
             Self::Data(d) => Ok(Type::Data(d.build(bytes)?)),
@@ -164,7 +174,7 @@ mod tests {
     #[test]
     fn test_typebuilder_build_valid() {
         let tb = TypeBuilder::Data(DataBuilder::String(StringDataBuilder {}));
-        let t = tb.build(b"hello, world!").unwrap();
+        let t = tb.build(Some(b"hello, world!")).unwrap();
         match t {
             Type::Data(Data::String(s)) => assert_eq!(s, "hello, world!".to_owned()),
             _ => panic!("Extracted type should have been a data string-type"),
@@ -175,14 +185,14 @@ mod tests {
     #[should_panic]
     fn test_typebuilder_build_invalid() {
         let tb = TypeBuilder::Data(DataBuilder::Bool(BoolDataBuilder {}));
-        tb.build(b"not a bool").unwrap();
+        tb.build(Some(b"not a bool")).unwrap();
     }
 
     #[test]
     fn test_typebuilder_from_typebuildercontainer_valid() {
         let tbc = TypeBuilderContainer(TypeBuilder::Data(DataBuilder::Bool(BoolDataBuilder {})));
         let tb: TypeBuilder = tbc.try_into().unwrap();
-        let t = tb.build(b"true").unwrap();
+        let t = tb.build(Some(b"true")).unwrap();
         match t {
             Type::Data(Data::Bool(b)) => assert_eq!(b, true),
             _ => panic!("Extracted data should have been a bool-type"),
